/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <app.h>
#include <appcore-efl.h>
#include <Ecore_X.h>
#include <aul.h>
#include <vconf.h>
#include <vconf-keys.h>
#include "phone.h"
#include "phone-common.h"
#include "ct-front.h"

#if !defined(C_PACKAGE)
#  define C_PACKAGE "contacts"
#endif

struct contact_data
{
	Evas_Object *win;
	ct_front_data *cfront_d;
	/* add more variables here */
#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
	Evas_Object *icon;
	Evas_Object *prev_btn;
#endif
};

static void ctapp_win_del(void *data, Evas_Object *obj, void *event)
{
	elm_exit();
}

static Evas_Object* ctapp_create_win(const char *name)
{
	Evas_Object *eo;
	int w, h;

	eo = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (eo) {
		elm_win_title_set(eo, name);
#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
#else
		elm_win_borderless_set(eo, EINA_TRUE);
#endif
		evas_object_smart_callback_add(eo, "delete,request", ctapp_win_del, NULL);
		ecore_x_window_size_get(ecore_x_window_root_first_get(), &w, &h);
		evas_object_resize(eo, w, h);
		elm_win_indicator_mode_set(eo, ELM_WIN_INDICATOR_SHOW);
	}
	elm_win_conformant_set(eo, EINA_TRUE);
	return eo;
}

static void ctapp_region_changed(void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	ct_front_data *cfront_d = ad->cfront_d;
	ctui_front_view_region_changed(cfront_d);
	ug_send_event(UG_EVENT_REGION_CHANGE);
	return;
}

static void ctapp_language_changed(void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	ct_front_data *cfront_d = ad->cfront_d;
	ctui_front_view_language_changed(cfront_d);
	ug_send_event(UG_EVENT_LANG_CHANGE);
	return;
}

static void device_orientation(app_device_orientation_e orientation, void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	ct_front_data *cfront_d = ad->cfront_d;
#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
	const char *config = elm_config_profile_get();
	if(!strcmp(config, "desktop"))
		return;
#endif
	p_retm_if(cfront_d == NULL || cfront_d->win == NULL, "Paramter is NULL");
	p_ret_if(!cfront_d->auto_rorate_screen);
	cfront_d->rotate = orientation;
	ctui_front_view_rotate(cfront_d, orientation);
	return;
}

static int ctapp_open(void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	ct_front_data *cfront_d = ad->cfront_d;
	if (EINA_FALSE == evas_object_visible_get(cfront_d->win)){
		evas_object_show(cfront_d->win);
		if(CT_VIEW_NONE == cfront_d->select_tab)
			ctui_front_view_reset(cfront_d, CT_VIEW_CONTACT, -1, NULL);
	}
	return 0;
}

#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
static Evas_Object* __set_win_icon(struct contact_data *ad)
{
	Evas_Object *image = evas_object_image_add(evas_object_evas_get(ad->win));
	evas_object_image_file_set(image, "/usr/share/icons/default/small/org.tizen.contacts.png", NULL);
	elm_win_icon_object_set(ad->win, image);

	return image;
}

static void __win_profile_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	if(data == NULL)
		return;

	struct contact_data *ad = data;

	const char *profile = elm_config_profile_get();
	if(!strcmp(profile, "desktop")) {
		Elm_Object_Item *navi_item = elm_naviframe_bottom_item_get(ad->cfront_d->navi);
		ad->prev_btn = elm_object_item_part_content_unset(navi_item, "prev_btn");
		elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_HIDE);
		if(!ad->icon)
			ad->icon = __set_win_icon(ad);
	}
	else {
		Elm_Object_Item *navi_item = elm_naviframe_bottom_item_get(ad->cfront_d->navi);
		if (ad->prev_btn) {
			elm_object_item_part_content_set(navi_item, "prev_btn", ad->prev_btn);
			ad->prev_btn = NULL;
		}
		elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	}
}
#endif //_PHONE_CONTACTS_DESKTOP_MODE_

static void __ctapp_auto_rotate_screen_bool_changed(keynode_t *key, void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;

	ct_front_data *cfront_d = ad->cfront_d;
	p_retm_if(cfront_d == NULL || cfront_d->win == NULL, "Paramter is NULL");

	vconf_get_bool(VCONFKEY_SETAPPL_AUTO_ROTATE_SCREEN_BOOL, &cfront_d->auto_rorate_screen);
	if (cfront_d->rotate != APP_DEVICE_ORIENTATION_0) {
		if (cfront_d->auto_rorate_screen)
			ctui_front_view_rotate(cfront_d, cfront_d->rotate);
		else
			ctui_front_view_rotate(cfront_d, APP_DEVICE_ORIENTATION_0);
	}
	return;
}

static bool ctapp_create(void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	Evas_Object *win;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	/* create window */
	win = ctapp_create_win(C_PACKAGE);
	p_retvm_if(NULL == win, false, "phapp_create_win() Failed");
	ad->win = win;


	bindtextdomain(PACKAGE, LOCALEDIR);
	err = contacts_connect2();
	if (CONTACTS_ERROR_NONE != err) {
		Evas_Object *popup;
		ERR("contacts_connect2() Failed(%d)", err);
		evas_object_show(win);
		popup = elm_popup_add(win);
		evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_show(popup);
		elm_object_text_set(popup, T_(PH_GET_TEXT_BASIC, PHTEXT_INITIALIZING));
		elm_popup_timeout_set(popup, 1.5);
		evas_object_smart_callback_add(popup, "timeout", ctapp_win_del, NULL);
		return true;
	}
#ifdef _PHONE_CONTACTS_DESKTOP_MODE_
	evas_object_smart_callback_add(ad->win, "profile,changed", __win_profile_changed_cb, ad);
#endif

	UG_INIT_EFL(ad->win, UG_OPT_INDICATOR_ENABLE);
	ad->cfront_d = ctui_front_view_create(ad->win);

	vconf_notify_key_changed(VCONFKEY_SETAPPL_AUTO_ROTATE_SCREEN_BOOL, __ctapp_auto_rotate_screen_bool_changed, ad);
	vconf_get_bool(VCONFKEY_SETAPPL_AUTO_ROTATE_SCREEN_BOOL, &(ad->cfront_d->auto_rorate_screen));

	ad->cfront_d->rotate = app_get_device_orientation();
	if (ad->cfront_d->auto_rorate_screen)
		elm_win_rotation_with_resize_set(ad->cfront_d->win, ad->cfront_d->rotate);
	appcore_set_open_cb(ctapp_open, data);
	return true;
}

static void ctapp_terminate(void *data)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	struct contact_data *ad = data;

	vconf_ignore_key_changed(VCONFKEY_SETAPPL_AUTO_ROTATE_SCREEN_BOOL, __ctapp_auto_rotate_screen_bool_changed);

	if (NULL != ad->win)
		evas_object_del(ad->win);
	//ug_destroy_all();
	err = contacts_disconnect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);
	return;
}

static void ctapp_pause(void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	ctui_front_view_pause(ad->cfront_d);
	ug_pause();
	return;
}

static void ctapp_resume(void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	ct_front_data *cfront_d = ad->cfront_d;
	if (CT_VIEW_NONE == cfront_d->select_tab)
		ctui_front_view_reset(cfront_d, CT_VIEW_CONTACT, -1, NULL);
	else
		ctui_front_view_resume(ad->cfront_d);
	ug_resume();
	return;
}

static void ctapp_service(service_h service, void *data)
{
	PH_FN_CALL;
	struct contact_data *ad = data;
	ct_front_data *cfront_d;
	int ret;
	char* val = NULL;

	p_retm_if(NULL == ad || NULL == ad->cfront_d, "data is null");
	cfront_d = ad->cfront_d;

	ret = service_get_extra_data(service, "launch_type", &val);
	p_warn_if(SERVICE_ERROR_NONE != ret, "service_get_extra_data is failed(%d)", ret);

	if (val && *val) {
		if (0 == strcmp(val, "favorite"))
			ctui_front_view_reset(cfront_d, CT_VIEW_FAVORITE, -1, NULL);
		else if (0 == strcmp(val, "groups"))
			ctui_front_view_reset(cfront_d, CT_VIEW_GROUP, -1, NULL);
		else if (0 == strcmp(val, "group_message"))
			ctui_front_view_reset(cfront_d, CT_VIEW_GROUP, CT_FRONT_GROUP_MESSAGE, service);
		else if (0 == strcmp(val, "group_email"))
			ctui_front_view_reset(cfront_d, CT_VIEW_GROUP, CT_FRONT_GROUP_EMAIL, service);
		else if (0 == strcmp(val, "group_detail"))
			ctui_front_view_reset(cfront_d, CT_VIEW_GROUP, CT_FRONT_GROUP_DETAIL, service);
		else if (0 == strcmp(val, "new"))
			ctui_front_view_reset(cfront_d, CT_VIEW_CONTACT, CT_FRONT_CONTACT_NEW, NULL);
		else if (0 == strcmp(val, "addrbook"))
			ctui_front_view_reset(cfront_d, CT_VIEW_CONTACT, CT_FRONT_CONTACT_ADDR, service);
		else if (0 == strcmp(val, "booting"))
			elm_win_lower(cfront_d->win);
		free(val);
		return;
	}

	ret = service_get_extra_data(service, "search_name", &val);
	p_warn_if(SERVICE_ERROR_NONE != ret, "service_get_extra_data is failed(%d)", ret);

	if (val && *val) {
		ctui_front_view_reset(cfront_d, CT_VIEW_CONTACT, CT_FRONT_CONTACT_SEARCH, service);
		free(val);
		return;
	}

	ctui_front_view_reset(cfront_d, CT_VIEW_CONTACT, -1, NULL);
	return;
}

API int main(int argc, char *argv[])
{
	struct contact_data ad = {0};
	app_event_callback_s event_callback = {0,};

	event_callback.create = ctapp_create;
	event_callback.terminate = ctapp_terminate;
	event_callback.pause = ctapp_pause;
	event_callback.resume = ctapp_resume;
	event_callback.service = ctapp_service;
	event_callback.low_memory = NULL;
	event_callback.low_battery = NULL;
	event_callback.device_orientation = device_orientation;
	event_callback.language_changed = ctapp_language_changed;
	event_callback.region_format_changed = ctapp_region_changed;

	return app_efl_main(&argc, &argv, &event_callback, &ad);
}

