/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <unicode/udatpg.h>
#include <unicode/ucal.h>
#include <unicode/ustring.h>
#include <ui-gadget-module.h>
#include <bundle.h>

#include "phone.h"
#include "ct-detail.h"
#include "ct-detail-utils.h"

#define KEY_SHARE_INFO_UG_DATA "cshare_info_d"

typedef struct {
	int person_id;
	ui_gadget_h ug;

	Evas_Object *navi;
	Evas_Object *genlist;
	Evas_Object *select_all_layout;

	UDateFormat* formatter;

	Eina_List *list_contacts;
}ct_share_info_ug_data;

typedef struct {
	contacts_record_h record;
	const char *view_uri;
	Eina_Bool checked;
}ct_value_info;

enum DETAIL_ITC_TYPE {
	CT_SHARE_INFO_UG_ITC_GROUP_TITLE,
	CT_SHARE_INFO_UG_ITC_NUMBER,
	CT_SHARE_INFO_UG_ITC_EMAIL,
	CT_SHARE_INFO_UG_ITC_ADDRESS,
	CT_SHARE_INFO_UG_ITC_MESSENGER,
	CT_SHARE_INFO_UG_ITC_WEBSITE,
	CT_SHARE_INFO_UG_ITC_EVENT,
	CT_SHARE_INFO_UG_ITC_NOTE,
};

static Elm_Genlist_Item_Class share_info_ug_itcs[] = {
	{.item_style="grouptitle"},
	{.item_style="2text.3"},
	{.item_style="2text.3"},
	{.item_style="2text.3"},
	{.item_style="2text.3"},
	{.item_style="2text.3"},
	{.item_style="2text.3"},
	{.item_style="2text.3"},
};

static char* share_info_ug_gl_group_title_label_get(void *data, Evas_Object *obj, const char *part)
{
	char *group_index = data;
	if (0 == strcmp(part, "elm.text") && group_index && *group_index)
		return strdup(group_index);
	else
		return NULL;
}

static char* share_info_ug_gl_number_label_get(void *data, Evas_Object *obj, const char *part)
{
	int type_int;
	char *number = NULL;
	char buf[PH_TEXT_MAX_LEN];
	ct_value_info *value_info = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	p_retv_if(NULL == value_info, NULL);

	if (0 == strcmp(part, "elm.text.2")) {
		err = contacts_record_get_int(value_info->record, _contacts_number.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		ctui_get_number_type_str(value_info->record, type_int, buf, sizeof(buf));
		return strdup(buf);
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		err = contacts_record_get_str_p(value_info->record, _contacts_number.number, &number);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		return SAFE_STRDUP(number);
	}
	return NULL;
}

static char* share_info_ug_gl_email_label_get(void *data, Evas_Object *obj, const char *part)
{
	int type_int;
	char *email = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_value_info *value_info = data;
	p_retv_if(NULL == value_info, NULL);

	if (0 == strcmp(part, "elm.text.2")) {
		err = contacts_record_get_int(value_info->record, _contacts_email.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		return strdup(T_(CT_GET_TEXT_EMAIL_TYPE, ctui_get_email_type_index(type_int)));
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		err = contacts_record_get_str_p(value_info->record, _contacts_email.email, &email);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
		return SAFE_STRDUP(email);
	}
	return NULL;
}

static char* share_info_ug_gl_address_label_get(void *data, Evas_Object *obj, const char *part)
{
	int type_int;
	char buf[PH_TEXT_MAX_LEN];
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_value_info *value_info = data;
	p_retv_if(NULL == value_info, NULL);

	if (0 == strcmp(part, "elm.text.2")) {
		err = contacts_record_get_int(value_info->record, _contacts_address.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		return strdup(T_(CT_GET_TEXT_POSTAL_TYPE, ct_detail_get_address_type_index(type_int)));
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		buf[0] = '\0';
		ct_detail_get_full_addr(value_info->record, buf, sizeof(buf));
		return elm_entry_utf8_to_markup(buf);
	}
	return NULL;
}

static char* share_info_ug_gl_messenger_label_get(void *data, Evas_Object *obj, const char *part)
{
	int type_int;
	char *im_id;
	ct_value_info *value_info = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	p_retv_if(NULL == value_info, NULL);

	if (0 == strcmp(part, "elm.text.2")) {
		err = contacts_record_get_int(value_info->record, _contacts_messenger.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		return strdup(T_(CT_GET_TEXT_IM_TYPE, type_int));
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		err = contacts_record_get_str_p(value_info->record, _contacts_messenger.im_id, &im_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
		return SAFE_STRDUP(im_id);
	}
	return NULL;
}

static char* share_info_ug_gl_website_label_get(void *data, Evas_Object *obj, const char *part)
{
	int type_int;
	char *url;
	ct_value_info *value_info = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	p_retv_if(NULL == value_info, NULL);

	if (0 == strcmp(part, "elm.text.2")) {
		err = contacts_record_get_int(value_info->record, _contacts_url.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		return strdup(T_(CT_GET_TEXT_WEBSITE_TYPE, ct_detail_get_website_type_index(type_int)));
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		err = contacts_record_get_str_p(value_info->record, _contacts_url.url, &url);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
		return SAFE_STRDUP(url);
	}
	return NULL;
}

static char* share_info_ug_gl_event_label_get(void *data, Evas_Object *obj, const char *part)
{
	int type_int;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_value_info *value_info = data;
	ct_share_info_ug_data *cshare_info_d;
	p_retv_if(NULL == value_info, NULL);

	cshare_info_d = evas_object_data_get(obj, KEY_SHARE_INFO_UG_DATA);
	p_retvm_if(NULL == cshare_info_d, NULL, "evas_object_data_get() return NULL");

	if (0 == strcmp(part, "elm.text.2")) {
		err = contacts_record_get_int(value_info->record, _contacts_url.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		return strdup(T_(CT_GET_TEXT_EVENT_TYPE, type_int));
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		int date;
		int day;
		int mon;
		int year;

		err = contacts_record_get_int(value_info->record, _contacts_event.date, &date);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		day = date%100;
		mon = (date%10000)/100;
		year = date/10000;

		p_retvm_if(mon < 1 || 12 < mon || day < 1 || 31 < day, NULL, "Invalid date(mon=%d,day=%d)",mon,day);
		return ct_detail_get_datetime_str(cshare_info_d->formatter, year, mon, day);
	}
	return NULL;
}

static char* share_info_ug_gl_note_label_get(void *data, Evas_Object *obj, const char *part)
{
	char *note;
	ct_value_info *value_info = data;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	p_retv_if(NULL == value_info, NULL);

	if (0 == strcmp(part, "elm.text.2")) {
		return strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_NOTE));
	}
	else if (0 == strcmp(part, "elm.text.1")) {
		err = contacts_record_get_str_p(value_info->record, _contacts_note.note, &note);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
		return SAFE_STRDUP(note);
	}
	return NULL;
}
static void share_info_ug_check_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	bool check_all;
	Evas_Object *check;
	Elm_Object_Item *item;
	Elm_Genlist_Item_Type type;
	ct_value_info *value_info;
	ct_share_info_ug_data *cshare_info_d = data;
	p_retm_if(NULL == cshare_info_d, "parameter(cshare_info_d) is NULL");

	check = elm_object_part_content_get(cshare_info_d->select_all_layout, "elm.icon");
	p_retm_if(NULL == check, "elm_object_part_content_get() return NULL");

	check_all = true;
	item = elm_genlist_first_item_get(cshare_info_d->genlist);
	while (item) {
		value_info = elm_object_item_data_get(item);
		type = elm_genlist_item_type_get(item);

		item = elm_genlist_item_next_get(item);
		if (ELM_GENLIST_ITEM_GROUP == type) continue;

		if (NULL == value_info) {
			ERR("elm_object_item_data_get() return NULL");
			continue;
		}
		if (!value_info->checked) {
			check_all = false;
			break;
		}
	}
	elm_check_state_set(check, check_all);
}

static Evas_Object* share_info_ug_gl_content_get(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *check;
	ct_value_info *value_info = data;
	ct_share_info_ug_data *cshare_info_d;

	cshare_info_d = evas_object_data_get(obj, KEY_SHARE_INFO_UG_DATA);
	p_retvm_if(NULL == cshare_info_d, NULL, "evas_object_data_get() return NULL");

	if (0 == strcmp(part, "elm.edit.icon.1")) {
		check = elm_check_add(obj);
		elm_check_state_pointer_set(check, &value_info->checked);
		evas_object_propagate_events_set(check, EINA_FALSE);
		evas_object_smart_callback_add(check, "changed", share_info_ug_check_changed_cb, cshare_info_d);
		return check;
	}
	return NULL;
}

static void share_info_ug_gl_del(void *data, Evas_Object *obj)
{
	ct_value_info *value_info = data;
	free(value_info);
}


static void share_info_ug_create_itcs()
{
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE].func.text_get = share_info_ug_gl_group_title_label_get;

	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NUMBER].func.text_get = share_info_ug_gl_number_label_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NUMBER].func.content_get = share_info_ug_gl_content_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NUMBER].func.del = share_info_ug_gl_del;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NUMBER].decorate_all_item_style = "edit_default";

	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EMAIL].func.text_get = share_info_ug_gl_email_label_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EMAIL].func.content_get = share_info_ug_gl_content_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EMAIL].func.del = share_info_ug_gl_del;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EMAIL].decorate_all_item_style = "edit_default";

	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_ADDRESS].func.text_get = share_info_ug_gl_address_label_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_ADDRESS].func.content_get = share_info_ug_gl_content_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_ADDRESS].func.del = share_info_ug_gl_del;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_ADDRESS].decorate_all_item_style = "edit_default";

	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_MESSENGER].func.text_get = share_info_ug_gl_messenger_label_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_MESSENGER].func.content_get = share_info_ug_gl_content_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_MESSENGER].func.del = share_info_ug_gl_del;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_MESSENGER].decorate_all_item_style = "edit_default";

	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_WEBSITE].func.text_get = share_info_ug_gl_website_label_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_WEBSITE].func.content_get = share_info_ug_gl_content_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_WEBSITE].func.del = share_info_ug_gl_del;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_WEBSITE].decorate_all_item_style = "edit_default";

	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EVENT].func.text_get = share_info_ug_gl_event_label_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EVENT].func.content_get = share_info_ug_gl_content_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EVENT].func.del = share_info_ug_gl_del;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EVENT].decorate_all_item_style = "edit_default";

	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NOTE].func.text_get = share_info_ug_gl_note_label_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NOTE].func.content_get = share_info_ug_gl_content_get;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NOTE].func.del = share_info_ug_gl_del;
	share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NOTE].decorate_all_item_style = "edit_default";

}

static void share_info_ug_sel(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *item = event_info;
	ct_value_info *value_info = data;
	ct_share_info_ug_data *cshare_info_d;
	elm_genlist_item_selected_set(item, EINA_FALSE);

	value_info->checked = !value_info->checked;
	elm_genlist_item_fields_update(item, "elm.edit.icon.1", ELM_GENLIST_ITEM_FIELD_CONTENT);

	cshare_info_d = evas_object_data_get(obj, KEY_SHARE_INFO_UG_DATA);
	share_info_ug_check_changed_cb(cshare_info_d, obj, NULL);
}

static void share_info_ug_fill_number_list(Evas_Object *genlist, Eina_List *list_contacts)
{
	Eina_List *l;
	Eina_List *list;
	contacts_record_h record = NULL;
	ct_value_info *value_info;

	list = ctui_person_get_number_list(list_contacts);
	p_retm_if(NULL == list, "ctui_person_get_number_list() return NULL");

	elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE],
			T_(CT_GET_TEXT_BASIC, CTTEXT_NUMBER), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	EINA_LIST_FOREACH(list, l, record) {
		value_info = calloc(1, sizeof(ct_value_info));
		if (NULL == value_info) {
			ERR("calloc() return NULL");
			continue;
		}
		value_info->view_uri = _contacts_number._uri;
		value_info->record = record;

		elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NUMBER],
				value_info, NULL, ELM_GENLIST_ITEM_NONE, share_info_ug_sel, value_info);
	}
	eina_list_free(list);
}

static void share_info_ug_fill_email_list(Evas_Object *genlist, Eina_List *list_contacts)
{
	Eina_List *l;
	Eina_List *list;
	contacts_record_h record = NULL;
	ct_value_info *value_info;

	list = ctui_person_get_email_list(list_contacts);
	p_retm_if(NULL == list, "ctui_person_get_email_list() return NULL");

	elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE],
			T_(CT_GET_TEXT_BASIC, CTTEXT_EMAIL), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	EINA_LIST_FOREACH(list, l, record) {
		value_info = calloc(1, sizeof(ct_value_info));
		if (NULL == value_info) {
			ERR("calloc() return NULL");
			continue;
		}
		value_info->view_uri = _contacts_email._uri;
		value_info->record = record;

		elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EMAIL],
				value_info, NULL, ELM_GENLIST_ITEM_NONE, share_info_ug_sel, value_info);
	}
	eina_list_free(list);
}

static void share_info_ug_fill_address_list(Evas_Object *genlist, Eina_List *list_contacts)
{
	Eina_List *l;
	Eina_List *list;
	contacts_record_h record = NULL;
	ct_value_info *value_info;

	list = ct_detail_person_get_postal_list(list_contacts);
	p_retm_if(NULL == list, "ct_detail_person_get_postal_list() return NULL");

	elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE],
			T_(CT_GET_TEXT_BASIC, CTTEXT_POSTAL_ADDRESS), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	EINA_LIST_FOREACH(list, l, record) {
		value_info = calloc(1, sizeof(ct_value_info));
		if (NULL == value_info) {
			ERR("calloc() return NULL");
			continue;
		}
		value_info->view_uri = _contacts_address._uri;
		value_info->record = record;

		elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_ADDRESS],
				value_info, NULL, ELM_GENLIST_ITEM_NONE, share_info_ug_sel, value_info);
	}
	eina_list_free(list);
}

static void share_info_ug_fill_messenger_list(Evas_Object *genlist, Eina_List *list_contacts)
{
	Eina_List *l;
	Eina_List *list;
	contacts_record_h record = NULL;
	ct_value_info *value_info;

	list = ct_detail_person_get_messenger_list(list_contacts);
	p_retm_if(NULL == list, "ct_detail_person_get_messenger_list() return NULL");

	elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE],
			T_(CT_GET_TEXT_BASIC, CTTEXT_INSTANT_MESSENGER), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	EINA_LIST_FOREACH(list, l, record) {
		value_info = calloc(1, sizeof(ct_value_info));
		if (NULL == value_info) {
			ERR("calloc() return NULL");
			continue;
		}
		value_info->view_uri = _contacts_messenger._uri;
		value_info->record = record;

		elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_MESSENGER],
				value_info, NULL, ELM_GENLIST_ITEM_NONE, share_info_ug_sel, value_info);
	}
	eina_list_free(list);
}

static void share_info_ug_fill_website_list(Evas_Object *genlist, Eina_List *list_contacts)
{
	Eina_List *l;
	Eina_List *list;
	contacts_record_h record = NULL;
	ct_value_info *value_info;

	list = ct_detail_person_get_website_list(list_contacts);
	p_retm_if(NULL == list, "ct_detail_person_get_website_list() return NULL");

	elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE],
			T_(CT_GET_TEXT_BASIC, CTTEXT_URL), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	EINA_LIST_FOREACH(list, l, record) {
		value_info = calloc(1, sizeof(ct_value_info));
		if (NULL == value_info) {
			ERR("calloc() return NULL");
			continue;
		}
		value_info->view_uri = _contacts_url._uri;
		value_info->record = record;

		elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_WEBSITE],
				value_info, NULL, ELM_GENLIST_ITEM_NONE, share_info_ug_sel, value_info);
	}
	eina_list_free(list);
}


static void share_info_ug_fill_event_list(Evas_Object *genlist, Eina_List *list_contacts)
{
	Eina_List *l;
	Eina_List *list;
	contacts_record_h record = NULL;
	ct_value_info *value_info;

	list = ct_detail_person_get_event_list(list_contacts);
	p_retm_if(NULL == list, "ct_detail_person_get_event_list() return NULL");

	elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE],
			T_(CT_GET_TEXT_BASIC, CTTEXT_EVENT), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	EINA_LIST_FOREACH(list, l, record) {
		value_info = calloc(1, sizeof(ct_value_info));
		if (NULL == value_info) {
			ERR("calloc() return NULL");
			continue;
		}
		value_info->view_uri = _contacts_event._uri;
		value_info->record = record;

		elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_EVENT],
				value_info, NULL, ELM_GENLIST_ITEM_NONE, share_info_ug_sel, value_info);
	}
	eina_list_free(list);
}

static void share_info_ug_fill_note_list(Evas_Object *genlist, Eina_List *list_contacts)
{
	bool group_title_added = false;
	Eina_List *l;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	contacts_record_h record_contact = NULL;
	ct_value_info *value_info;

	EINA_LIST_FOREACH(list_contacts, l, record_contact) {
		if (NULL == record_contact) continue;

		err = contacts_record_get_child_record_at_p(record_contact, _contacts_contact.note, 0, &record);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_record_get_child_record_at_p() Failed(%d)", err);
			continue;
		}

		value_info = calloc(1, sizeof(ct_value_info));
		if (NULL == value_info) {
			ERR("calloc() return NULL");
			continue;
		}
		value_info->view_uri = _contacts_note._uri;
		value_info->record = record;

		if (!group_title_added) {
			elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_GROUP_TITLE],
					T_(CT_GET_TEXT_BASIC, CTTEXT_NOTE), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
			group_title_added = true;
		}

		elm_genlist_item_append(genlist, &share_info_ug_itcs[CT_SHARE_INFO_UG_ITC_NOTE],
				value_info, NULL, ELM_GENLIST_ITEM_NONE, share_info_ug_sel, value_info);
	}
}

static void share_info_ug_fill_layout(ct_share_info_ug_data *cshare_info_d, int person_id)
{
	Eina_List *list_contacts;
	Evas_Object *genlist;

	p_retm_if(NULL == cshare_info_d, "parameter(cshare_info_d) is NULL");
	genlist = cshare_info_d->genlist;

	cshare_info_d->person_id = person_id;
	cshare_info_d->formatter = ct_detail_init_time_formatter();

	list_contacts = ctui_person_get_contact_list(person_id);
	p_retm_if(NULL == list_contacts, "ctui_person_get_contact_list() return NULL");
	cshare_info_d->list_contacts = list_contacts;

	share_info_ug_create_itcs();
	share_info_ug_fill_number_list(genlist, list_contacts);
	share_info_ug_fill_email_list(genlist, list_contacts);
	share_info_ug_fill_address_list(genlist, list_contacts);
	share_info_ug_fill_messenger_list(genlist, list_contacts);
	share_info_ug_fill_website_list(genlist, list_contacts);
	share_info_ug_fill_event_list(genlist, list_contacts);
	share_info_ug_fill_note_list(genlist, list_contacts);
}

static void share_info_ug_delete_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	ct_share_info_ug_data *cshare_info_d = data;
	p_retm_if(NULL == cshare_info_d, "paramter(cshare_info_d) is NULL");

	if (cshare_info_d->formatter)
		udat_close(cshare_info_d->formatter);

	ctui_free_record_list(cshare_info_d->list_contacts);

	free(cshare_info_d);
}

static void share_info_ug_append_value_str(ct_value_info *value_info, UDateFormat* formatter, char *buf, int buf_len)
{
	int date;
	int day;
	int mon;
	int year;
	int type_int;
	char *text;
	char *date_str;
	char temp[PH_TEXT_MAX_LEN];
	char temp_str[PH_TEXT_MAX_LEN];
	contacts_error_e err = CONTACTS_ERROR_NONE;

	p_retm_if(NULL == value_info, "parameter(value_info) is NULL");

	if (0 == strcmp(value_info->view_uri, _contacts_number._uri)) {
		temp_str[0] = '\0';

		err = contacts_record_get_int(value_info->record, _contacts_number.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		ctui_get_number_type_str(value_info->record, type_int, temp_str, sizeof(temp_str));

		err = contacts_record_get_str_p(value_info->record, _contacts_number.number, &text);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		snprintf(temp, sizeof(temp), "%s%s\n%s\n", buf, temp_str, SAFE_STR(text));
		snprintf(buf, buf_len, "%s", temp);
	}
	else if (0 == strcmp(value_info->view_uri, _contacts_email._uri)) {
		temp_str[0] = '\0';

		err = contacts_record_get_int(value_info->record, _contacts_email.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		err = contacts_record_get_str_p(value_info->record, _contacts_email.email, &text);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		snprintf(temp, sizeof(temp), "%s%s\n%s\n", buf, T_(CT_GET_TEXT_EMAIL_TYPE, ctui_get_email_type_index(type_int)), SAFE_STR(text));
		snprintf(buf, buf_len, "%s", temp);
	}
	else if (0 == strcmp(value_info->view_uri, _contacts_address._uri)) {
		temp_str[0] = '\0';

		err = contacts_record_get_int(value_info->record, _contacts_address.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		ct_detail_get_full_addr(value_info->record, temp_str, sizeof(temp_str));

		snprintf(temp, sizeof(temp), "%s%s\n%s\n", buf, T_(CT_GET_TEXT_POSTAL_TYPE, ct_detail_get_address_type_index(type_int)), temp_str);
		snprintf(buf, buf_len, "%s", temp);
	}
	else if (0 == strcmp(value_info->view_uri, _contacts_messenger._uri)) {
		temp_str[0] = '\0';

		err = contacts_record_get_int(value_info->record, _contacts_messenger.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		err = contacts_record_get_str_p(value_info->record, _contacts_messenger.im_id, &text);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		snprintf(temp, sizeof(temp), "%s%s\n%s\n", buf, T_(CT_GET_TEXT_IM_TYPE, type_int), SAFE_STR(text));
		snprintf(buf, buf_len, "%s", temp);
	}
	else if (0 == strcmp(value_info->view_uri, _contacts_url._uri)) {
		temp_str[0] = '\0';

		err = contacts_record_get_int(value_info->record, _contacts_url.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		err = contacts_record_get_str_p(value_info->record, _contacts_url.url, &text);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		snprintf(temp, sizeof(temp), "%s%s\n%s\n", buf, T_(CT_GET_TEXT_WEBSITE_TYPE, ct_detail_get_website_type_index(type_int)), SAFE_STR(text));
		snprintf(buf, buf_len, "%s", temp);
	}
	else if (0 == strcmp(value_info->view_uri, _contacts_event._uri)) {
		temp_str[0] = '\0';

		err = contacts_record_get_int(value_info->record, _contacts_event.type, &type_int);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		err = contacts_record_get_int(value_info->record, _contacts_event.date, &date);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

		day = date%100;
		mon = (date%10000)/100;
		year = date/10000;

		date_str = ct_detail_get_datetime_str(formatter, year, mon, day);
		snprintf(temp, sizeof(temp), "%s%s\n%s\n", buf, T_(CT_GET_TEXT_EVENT_TYPE, type_int), SAFE_STR(date_str));
		snprintf(buf, buf_len, "%s", temp);
		free(date_str);
	}
	else if (0 == strcmp(value_info->view_uri, _contacts_note._uri)) {
		temp_str[0] = '\0';

		err = contacts_record_get_str_p(value_info->record, _contacts_note.note, &text);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

		snprintf(temp, sizeof(temp), "%s%s\n%s\n", buf, T_(CT_GET_TEXT_BASIC, CTTEXT_NOTE), SAFE_STR(text));
		snprintf(buf, buf_len, "%s", temp);
	}
}

static void share_info_ug_get_person_str(Evas_Object *genlist, UDateFormat* formatter, char *buf, int buf_len)
{
	Elm_Object_Item *item;
	Elm_Genlist_Item_Type type;
	ct_value_info *value_info;

	p_retm_if(NULL == genlist, "parameter(genlist) is NULL");
	p_retm_if(NULL == buf, "parameter(buf) is NULL");

	item = elm_genlist_first_item_get(genlist);
	while (item) {
		value_info = elm_object_item_data_get(item);
		type = elm_genlist_item_type_get(item);

		item = elm_genlist_item_next_get(item);
		if (ELM_GENLIST_ITEM_GROUP == type) continue;

		if (NULL == value_info) {
			ERR("elm_object_item_data_get() return NULL");
			continue;
		}

		if (value_info->checked)
			share_info_ug_append_value_str(value_info, formatter, buf, buf_len);
	}
	return;
}

static void share_info_ug_get_person_display_name(int person_id, char *buf, int buf_len)
{
	char *display_name;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	err = contacts_db_get_record(_contacts_person._uri, person_id, &record);
	p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

	err = contacts_record_get_str_p(record, _contacts_person.display_name, &display_name);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
	snprintf(buf, buf_len, "%s\n", SAFE_STR(display_name));

	err = contacts_record_destroy(record, true);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);
}

static void share_info_ug_save_cb(void *data, Evas_Object *obj, void *event_info)
{
	char buf[PH_TEXT_MAX_LEN*10];
	service_h service;
	ct_share_info_ug_data *cshare_info_d = data;
	p_retm_if(NULL == cshare_info_d, "parameter(ct_share_info_ug_data) is NULL");
	service_create(&service);

	buf[0] = '\0';
	share_info_ug_get_person_display_name(cshare_info_d->person_id, buf, sizeof(buf));
	share_info_ug_get_person_str(cshare_info_d->genlist, cshare_info_d->formatter, buf, sizeof(buf));
	if (*buf)
		service_add_extra_data(service, CT_UG_BUNDLE_RESULT_PERSON_INFO, buf);
	ug_send_result(cshare_info_d->ug, service);
	service_destroy(service);
	ug_destroy_me(cshare_info_d->ug);
}

static void share_info_ug_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *navi = data;
	elm_naviframe_item_pop(navi);
}

static void share_info_ug_select_all_changed_cb(void *data, Eina_Bool checked)
{
	Elm_Object_Item *item;
	Elm_Genlist_Item_Type type;
	ct_value_info *value_info;
	ct_share_info_ug_data *cshare_info_d = data;

	item = elm_genlist_first_item_get(cshare_info_d->genlist);
	while (item) {
		value_info = elm_object_item_data_get(item);
		type = elm_genlist_item_type_get(item);

		item = elm_genlist_item_next_get(item);
		if (ELM_GENLIST_ITEM_GROUP == type) continue;

		if (NULL == value_info) {
			ERR("elm_object_item_data_get() return NULL");
			continue;
		}
		value_info->checked = checked;
	}
	elm_genlist_realized_items_update(cshare_info_d->genlist);
}

API void ctui_create_share_info_ug_layout(Evas_Object *navi, int person_id, ui_gadget_h ug)
{
	Evas_Object *btn;
	Evas_Object *base;
	Evas_Object *box;
	Evas_Object *genlist;
	Evas_Object *toolbar;
	Evas_Object *select_all_layout;
	Evas_Object *done_button;
	Elm_Object_Item *navi_item;
	ct_share_info_ug_data *cshare_info_d;

	base = phone_create_base_layout(navi, false);
	evas_object_show(base);

	box = elm_box_add(base);
	p_retm_if(NULL == box, "elm_box_add() return NULL");
	evas_object_show(box);

	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_FILL);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_content_set(base, "elm.swallow.content", box);

	genlist = elm_genlist_add(box);
	p_retm_if(NULL == genlist, "elm_genlist_add() return NULL");
	evas_object_show(genlist);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_pack_end(box, genlist);
	elm_genlist_decorate_mode_set(genlist, EINA_TRUE);

	cshare_info_d = calloc(1, sizeof(ct_share_info_ug_data));
	p_retm_if(NULL == cshare_info_d, "calloc() return NULL");
	cshare_info_d->ug = ug;
	cshare_info_d->navi = navi;
	cshare_info_d->genlist = genlist;
	evas_object_data_set(genlist, KEY_SHARE_INFO_UG_DATA, cshare_info_d);

	share_info_ug_fill_layout(cshare_info_d, person_id);
	evas_object_event_callback_add(base, EVAS_CALLBACK_DEL, share_info_ug_delete_cb, cshare_info_d);

	select_all_layout = phone_create_select_all(box, S_(PH_SYS_BODY_SELECT_ALL), share_info_ug_select_all_changed_cb, cshare_info_d);
	elm_box_pack_start(box, select_all_layout);
	cshare_info_d->select_all_layout = select_all_layout;

	navi_item = elm_naviframe_item_push(navi, T_(CT_GET_TEXT_BASIC, CTTEXT_SHARE_CONTACT_INFO), NULL, NULL, base, NULL);
	toolbar = elm_toolbar_add(navi);
	elm_toolbar_transverse_expanded_set(toolbar, EINA_TRUE);
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_object_item_part_content_set(navi_item , "controlbar", toolbar);

	btn = elm_button_add(navi);
	elm_object_style_set(btn, "naviframe/back_btn/default");
	evas_object_smart_callback_add(btn, "clicked", share_info_ug_cancel_cb, cshare_info_d->navi);
	elm_object_item_part_content_set(navi_item, "prev_btn", btn);

	done_button = phone_create_toolbar_btn(navi, S_(CT_SYS_SK_DONE), share_info_ug_save_cb, cshare_info_d);
	elm_object_item_part_content_set(navi_item, "toolbar_button1", done_button);
}

